/*
 * Copyright (c) 2004-2016 Zementis, Inc.
 * Copyright (c) 2016-2017 Software AG, Darmstadt, Germany and/or Software AG USA Inc., Reston, VA, USA, and/or its 
 * subsidiaries and/or its affiliates and/or their licensors.
 * Use, reproduction, transfer, publication or disclosure is prohibited except as specifically provided for in your 
 * License Agreement with Software AG.
 */
package com.zementis.adapa.sample.lookuptables;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Arrays;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.sql.DataSource;

import com.mysql.jdbc.jdbc2.optional.MysqlDataSource;

import com.zementis.adapa.extensions.LookupTable;

/**
 * This lookup table returns a GDP number corresponding to Country and State. Country, State and GDP are columns in the
 * database table GDP_Table.
 * 
 */
public class GDPLookupTable implements LookupTable {

    private static final Logger LOGGER = Logger.getLogger(GDPLookupTable.class.getName());

    private static final String DB_TABLE_NAME = "GDP_Table";

    private static final String LOOKUP_TABLE_NAME = "GDP_Lookup";

    private static final String DATABASE_NAME = "lookup";

    private static final String[] INPUT_COLUMN_NAMES = { "Country", "State", };

    private static final String OUTPUT_COLUMN_NAME = "GDP";

    private static final DataSource DATASOURCE = createDataSource();

    /**
     * @return {@link DataSource}
     */
    protected static DataSource createDataSource() {
        LOGGER.info("Creating MySQL data source...");
        MysqlDataSource dataSource = new MysqlDataSource();
        dataSource.setServerName("localhost");
        dataSource.setUser("root");
        dataSource.setDatabaseName(DATABASE_NAME);
        return dataSource;
    }

    @Override
    public boolean canLookUpNullValues() {
        return false;
    }

    @Override
    public Class<?> getColumnClass(String columnName) {
        try {
            LOGGER.info("Connecting to MySQL database...");
            Connection connection = DATASOURCE.getConnection();
            DatabaseMetaData metadata = connection.getMetaData();
            ResultSet resultSet = metadata.getColumns(null, null, DB_TABLE_NAME, columnName);
            LOGGER.info("Table " + DB_TABLE_NAME + " contains :" + resultSet.toString());
            while (resultSet.next()) {
                return getJavaType(resultSet.getString("TYPE_NAME"));
            }
            resultSet.close();
            connection.close();
        } catch (SQLException sqle) {
            LOGGER.log(Level.SEVERE, sqle.getMessage(), sqle);
        }
        return null;
    }

    @Override
    public List<String> getInputColumnNames() {
        return Arrays.asList(INPUT_COLUMN_NAMES);
    }

    @Override
    public String getName() {
        return LOOKUP_TABLE_NAME;
    }

    @Override
    public String getOutputColumnName() {
        return OUTPUT_COLUMN_NAME;
    }

    @Override
    public Object lookup(Object... inputs) {
        if (inputs.length != 2) {
            return null;
        }
        if (inputs[0] == null || inputs[1] == null) {
            return null;
        }
        try {
            Connection connection = DATASOURCE.getConnection();
            String sqlQuery =
                "SELECT " + OUTPUT_COLUMN_NAME + " FROM " + DB_TABLE_NAME + " WHERE " + INPUT_COLUMN_NAMES[0]
                    + " = \"" + inputs[0] + "\" and " + INPUT_COLUMN_NAMES[1] + " = \"" + inputs[1] + "\"";
            ResultSet resultSet = connection.prepareStatement(sqlQuery).executeQuery();
            if (resultSet.next()) {
                return resultSet.getObject(1);
            }
            resultSet.close();
            connection.close();
        } catch (SQLException sqle) {
            LOGGER.log(Level.SEVERE, sqle.getMessage(), sqle);
        }
        return null;
    }

    /**
     * @param sqlType
     *            SQL Type
     * @return Java Type
     */
    private Class<Double> getJavaType(String sqlType) {
        // return appropriate Java type based on the provided SQL Type
        return Double.class;
    }
}
